##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################


"""
HTTPMounter
$Id: HTTPMounter.py,v 1.14 2002/05/31 15:34:38 andreasjung Exp $
"""

from Globals import Persistent,DTMLFile
from OFS.SimpleItem import SimpleItem
from Acquisition import Implicit
import string, time, urlparse, httplib, re
from zLOG import INFO, LOG

MAX_TRIES = 5        # max. number of retries
TIME2SLEEP = 0.5     # time to sleep in seconds between retries

class HTTPMounter(Persistent, Implicit, SimpleItem):
    """ The HTTPMounter product acts like a dumb LocalFS and mounts
        another HTTP server into a Zope instance.

        Accessing content directly through direct traversal:

           /foo/http_mounter_mountpoint/relative/remote/path/bar.txt

       or inside an acquired object

           /foo/http_mounter_mountpoint/relative/remote/path/bar.txt/render_html

          where render_html might containt <dtml-var getContent> to access
          the content of bar.txt

    """

    meta_type = 'HTTPMounter'
    default_document = 'index.html'

    manage_options= (
        {'label': 'Settings',     
         'action': 'manage_workspace'
         },
        {'label': 'View',     
         'action': '' 
         },
    )

    def __init__(self, id, title, url): 
        self.id     = id
        self.title  = title
        self.url    = url
        self._v_traversal = []  # keep path information relative to the mountpoint

    def __bobo_traverse__(self, request, entry_name):
        """ fool Zope traversal - hehehe"""

        # check if we have the relative traversal stack
        if not hasattr(self, '_v_traversal'):  
            self._v_traversal = []

        # check for our ZMI management methods
        if entry_name in ('manage_main','manage_workspace', 'manage_preferences'):
            return getattr(self, entry_name)

        if hasattr(self, entry_name): 

            # We need to return an acquired object !
            # So we first retrieve the object we collected
            # traversal information from.

            tup  = urlparse.urlparse(self.url)
            host = tup[1]
            path = tup[2] 
            if path[-1] != '/': path = path + '/'
            path = path + string.join(self._v_traversal, '/')   

            tries = 0
            while tries < MAX_TRIES:

                try:
                    self._http_request(host, path)          
                    self._v_traversal = []
                    return getattr(self, entry_name)

                except IOError:
                    tries = tries + 1
                    time.sleep(TIME2SLEEP)
                 
            raise IOError

        else:

            # we don't have acquire an object, so we 
            # collect the traversal information until we need it.
            
            self._v_traversal.append(entry_name)    
            return self


    def __call__(self, REQUEST, RESPONSE):
        """ """

        own_path = string.join(self.getPhysicalPath(), '/')

        path = REQUEST['URL1']
        pos = string.rfind(path, self.id)
        sub_path = path[pos + len(self.id):]

        if REQUEST['URL1'][- len(self.id):] == self.id:
            sub_path = sub_path + self.default_document

        tries = 0

        while tries<MAX_TRIES:
            try:        
                tup  = urlparse.urlparse(self.url)
                host = tup[1]
                path = tup[2] 
                if path[-1] != '/': path = path + '/'
                path = path + sub_path

                status, data, headers = self._http_request(host, path)

                # get headers from original response and fake them into
                # our response to the web client
                for k in ('content-type','content-length','etag','last-modified'):
                    v = headers.getheader(k)
                    if v: RESPONSE.setHeader(k,v)

                RESPONSE.write(data)           
                return

            except IOError:
                tries = tries + 1
                time.sleep(TIME2SLEEP)

        return IOError

    index_html = __call__

    def _http_request(self, host, path):
        """ get URL """
                            
        H = httplib.HTTP(host) 
        H.putrequest('GET', path)
        H.putheader('Accept','text/plain')
        H.putheader('Accept','text/html')
        H.putheader('Accept','image/jpeg')
        H.putheader('Accept','image/gif')
        H.endheaders()

        status, msg, headers = H.getreply()
        self._v_content = H.getfile().read()

        return (status, self._v_content, headers)

    def getContent(self):
        """ returns content"""
        return self._v_content


    def manage_preferences(self, title, url, default_document, RESPONSE=None, URL1=None):
        """ save preferences """

        self.title = title
        self.url   = url
        self.default_document = default_document

        RESPONSE.redirect(URL1 + '/manage_main?manage_tabs_message=Settings%20saved')



    manage_workspace  = DTMLFile("dtml/manageHTTPMounter",globals())


manage_addHTTPMounterForm = DTMLFile('dtml/addHTTPMounter', globals())

def manage_addHTTPMounter(self, id, title, url, REQUEST=None, RESPONSE=None, URL3=None):
    """Add a new HTTPMounter """

    mounter = HTTPMounter(id, title, url)
    self._setObject(id, mounter)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST,update_menu=1)
    
